# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import field_mask_pb2  # type: ignore
import proto  # type: ignore

from google.cloud.cloudquotas_v1.types import resources

__protobuf__ = proto.module(
    package="google.api.cloudquotas.v1",
    manifest={
        "ListQuotaInfosRequest",
        "ListQuotaInfosResponse",
        "GetQuotaInfoRequest",
        "ListQuotaPreferencesRequest",
        "ListQuotaPreferencesResponse",
        "GetQuotaPreferenceRequest",
        "CreateQuotaPreferenceRequest",
        "UpdateQuotaPreferenceRequest",
    },
)


class ListQuotaInfosRequest(proto.Message):
    r"""Message for requesting list of QuotaInfos

    Attributes:
        parent (str):
            Required. Parent value of QuotaInfo resources. Listing
            across different resource containers (such as 'projects/-')
            is not allowed.

            Example names:
            ``projects/123/locations/global/services/compute.googleapis.com``
            ``folders/234/locations/global/services/compute.googleapis.com``
            ``organizations/345/locations/global/services/compute.googleapis.com``
        page_size (int):
            Optional. Requested page size. Server may
            return fewer items than requested. If
            unspecified, server will pick an appropriate
            default.
        page_token (str):
            Optional. A token identifying a page of
            results the server should return.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )


class ListQuotaInfosResponse(proto.Message):
    r"""Message for response to listing QuotaInfos

    Attributes:
        quota_infos (MutableSequence[google.cloud.cloudquotas_v1.types.QuotaInfo]):
            The list of QuotaInfo
        next_page_token (str):
            A token, which can be sent as ``page_token`` to retrieve the
            next page. If this field is omitted, there are no subsequent
            pages.
    """

    @property
    def raw_page(self):
        return self

    quota_infos: MutableSequence[resources.QuotaInfo] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=resources.QuotaInfo,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class GetQuotaInfoRequest(proto.Message):
    r"""Message for getting a QuotaInfo

    Attributes:
        name (str):
            Required. The resource name of the quota info.

            An example name:
            ``projects/123/locations/global/services/compute.googleapis.com/quotaInfos/CpusPerProjectPerRegion``
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ListQuotaPreferencesRequest(proto.Message):
    r"""Message for requesting list of QuotaPreferences

    Attributes:
        parent (str):
            Required. Parent value of QuotaPreference resources. Listing
            across different resource containers (such as 'projects/-')
            is not allowed.

            When the value starts with 'folders' or 'organizations', it
            lists the QuotaPreferences for org quotas in the container.
            It does not list the QuotaPreferences in the descendant
            projects of the container.

            Example parents: ``projects/123/locations/global``
        page_size (int):
            Optional. Requested page size. Server may
            return fewer items than requested. If
            unspecified, server will pick an appropriate
            default.
        page_token (str):
            Optional. A token identifying a page of
            results the server should return.
        filter (str):
            Optional. Filter result QuotaPreferences by their state,
            type, create/update time range.

            Example filters:
            ``reconciling=true AND request_type=CLOUD_CONSOLE``,
            ``reconciling=true OR creation_time>2022-12-03T10:30:00``
        order_by (str):
            Optional. How to order of the results. By default, the
            results are ordered by create time.

            Example orders: ``quota_id``, ``service, create_time``
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )
    filter: str = proto.Field(
        proto.STRING,
        number=4,
    )
    order_by: str = proto.Field(
        proto.STRING,
        number=5,
    )


class ListQuotaPreferencesResponse(proto.Message):
    r"""Message for response to listing QuotaPreferences

    Attributes:
        quota_preferences (MutableSequence[google.cloud.cloudquotas_v1.types.QuotaPreference]):
            The list of QuotaPreference
        next_page_token (str):
            A token, which can be sent as ``page_token`` to retrieve the
            next page. If this field is omitted, there are no subsequent
            pages.
        unreachable (MutableSequence[str]):
            Locations that could not be reached.
    """

    @property
    def raw_page(self):
        return self

    quota_preferences: MutableSequence[resources.QuotaPreference] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=resources.QuotaPreference,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )
    unreachable: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=3,
    )


class GetQuotaPreferenceRequest(proto.Message):
    r"""Message for getting a QuotaPreference

    Attributes:
        name (str):
            Required. Name of the resource

            Example name:
            ``projects/123/locations/global/quota_preferences/my-config-for-us-east1``
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class CreateQuotaPreferenceRequest(proto.Message):
    r"""Message for creating a QuotaPreference

    Attributes:
        parent (str):
            Required. Value for parent.

            Example: ``projects/123/locations/global``
        quota_preference_id (str):
            Optional. Id of the requesting object, must
            be unique under its parent. If client does not
            set this field, the service will generate one.
        quota_preference (google.cloud.cloudquotas_v1.types.QuotaPreference):
            Required. The resource being created
        ignore_safety_checks (MutableSequence[google.cloud.cloudquotas_v1.types.QuotaSafetyCheck]):
            The list of quota safety checks to be
            ignored.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    quota_preference_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    quota_preference: resources.QuotaPreference = proto.Field(
        proto.MESSAGE,
        number=3,
        message=resources.QuotaPreference,
    )
    ignore_safety_checks: MutableSequence[
        resources.QuotaSafetyCheck
    ] = proto.RepeatedField(
        proto.ENUM,
        number=4,
        enum=resources.QuotaSafetyCheck,
    )


class UpdateQuotaPreferenceRequest(proto.Message):
    r"""Message for updating a QuotaPreference

    Attributes:
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Optional. Field mask is used to specify the fields to be
            overwritten in the QuotaPreference resource by the update.
            The fields specified in the update_mask are relative to the
            resource, not the full request. A field will be overwritten
            if it is in the mask. If the user does not provide a mask
            then all fields will be overwritten.
        quota_preference (google.cloud.cloudquotas_v1.types.QuotaPreference):
            Required. The resource being updated
        allow_missing (bool):
            Optional. If set to true, and the quota preference is not
            found, a new one will be created. In this situation,
            ``update_mask`` is ignored.
        validate_only (bool):
            Optional. If set to true, validate the
            request, but do not actually update. Note that a
            request being valid does not mean that the
            request is guaranteed to be fulfilled.
        ignore_safety_checks (MutableSequence[google.cloud.cloudquotas_v1.types.QuotaSafetyCheck]):
            The list of quota safety checks to be
            ignored.
    """

    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=1,
        message=field_mask_pb2.FieldMask,
    )
    quota_preference: resources.QuotaPreference = proto.Field(
        proto.MESSAGE,
        number=2,
        message=resources.QuotaPreference,
    )
    allow_missing: bool = proto.Field(
        proto.BOOL,
        number=3,
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=4,
    )
    ignore_safety_checks: MutableSequence[
        resources.QuotaSafetyCheck
    ] = proto.RepeatedField(
        proto.ENUM,
        number=5,
        enum=resources.QuotaSafetyCheck,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import timestamp_pb2  # type: ignore
from google.protobuf import wrappers_pb2  # type: ignore
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.api.cloudquotas.v1",
    manifest={
        "QuotaSafetyCheck",
        "QuotaInfo",
        "QuotaIncreaseEligibility",
        "QuotaPreference",
        "QuotaConfig",
        "DimensionsInfo",
        "QuotaDetails",
    },
)


class QuotaSafetyCheck(proto.Enum):
    r"""Enumerations of quota safety checks.

    Values:
        QUOTA_SAFETY_CHECK_UNSPECIFIED (0):
            Unspecified quota safety check.
        QUOTA_DECREASE_BELOW_USAGE (1):
            Validates that a quota mutation would not
            cause the consumer's effective limit to be lower
            than the consumer's quota usage.
        QUOTA_DECREASE_PERCENTAGE_TOO_HIGH (2):
            Validates that a quota mutation would not
            cause the consumer's effective limit to decrease
            by more than 10 percent.
    """
    QUOTA_SAFETY_CHECK_UNSPECIFIED = 0
    QUOTA_DECREASE_BELOW_USAGE = 1
    QUOTA_DECREASE_PERCENTAGE_TOO_HIGH = 2


class QuotaInfo(proto.Message):
    r"""QuotaInfo represents information about a particular quota for
    a given project, folder or organization.

    Attributes:
        name (str):
            Resource name of this QuotaInfo. The ID component following
            "locations/" must be "global". Example:
            ``projects/123/locations/global/services/compute.googleapis.com/quotaInfos/CpusPerProjectPerRegion``
        quota_id (str):
            The id of the quota, which is unquie within the service.
            Example: ``CpusPerProjectPerRegion``
        metric (str):
            The metric of the quota. It specifies the resources
            consumption the quota is defined for. Example:
            ``compute.googleapis.com/cpus``
        service (str):
            The name of the service in which the quota is defined.
            Example: ``compute.googleapis.com``
        is_precise (bool):
            Whether this is a precise quota. A precise
            quota is tracked with absolute precision. In
            contrast, an imprecise quota is not tracked with
            precision.
        refresh_interval (str):
            The reset time interval for the quota.
            Refresh interval applies to rate quota only.
            Example: "minute" for per minute, "day" for per
            day, or "10 seconds" for every 10 seconds.
        container_type (google.cloud.cloudquotas_v1.types.QuotaInfo.ContainerType):
            The container type of the QuotaInfo.
        dimensions (MutableSequence[str]):
            The dimensions the quota is defined on.
        metric_display_name (str):
            The display name of the quota metric
        quota_display_name (str):
            The display name of the quota.
        metric_unit (str):
            The unit in which the metric value is
            reported, e.g., "MByte".
        quota_increase_eligibility (google.cloud.cloudquotas_v1.types.QuotaIncreaseEligibility):
            Whether it is eligible to request a higher
            quota value for this quota.
        is_fixed (bool):
            Whether the quota value is fixed or
            adjustable
        dimensions_infos (MutableSequence[google.cloud.cloudquotas_v1.types.DimensionsInfo]):
            The collection of dimensions info ordered by
            their dimensions from more specific ones to less
            specific ones.
        is_concurrent (bool):
            Whether the quota is a concurrent quota.
            Concurrent quotas are enforced on the total
            number of concurrent operations in flight at any
            given time.
        service_request_quota_uri (str):
            URI to the page where the user can request more quotas for
            the cloud service, such as
            https://docs.google.com/spreadsheet/viewform?formkey=abc123&entry_0={email}&entry_1={id}.
            Google Developers Console UI replace {email} with the
            current user's e-mail, {id} with the current project number,
            or organization ID with "organizations/" prefix. For
            example,
            https://docs.google.com/spreadsheet/viewform?formkey=abc123&entry_0=johndoe@gmail.com&entry_1=25463754,
            or
            https://docs.google.com/spreadsheet/viewform?formkey=abc123&entry_0=johndoe@gmail.com&entry_1=organizations/26474422.
    """

    class ContainerType(proto.Enum):
        r"""The enumeration of the types of a cloud resource container.

        Values:
            CONTAINER_TYPE_UNSPECIFIED (0):
                Unspecified container type.
            PROJECT (1):
                consumer project
            FOLDER (2):
                folder
            ORGANIZATION (3):
                organization
        """
        CONTAINER_TYPE_UNSPECIFIED = 0
        PROJECT = 1
        FOLDER = 2
        ORGANIZATION = 3

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    quota_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    metric: str = proto.Field(
        proto.STRING,
        number=3,
    )
    service: str = proto.Field(
        proto.STRING,
        number=4,
    )
    is_precise: bool = proto.Field(
        proto.BOOL,
        number=5,
    )
    refresh_interval: str = proto.Field(
        proto.STRING,
        number=6,
    )
    container_type: ContainerType = proto.Field(
        proto.ENUM,
        number=7,
        enum=ContainerType,
    )
    dimensions: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=8,
    )
    metric_display_name: str = proto.Field(
        proto.STRING,
        number=9,
    )
    quota_display_name: str = proto.Field(
        proto.STRING,
        number=10,
    )
    metric_unit: str = proto.Field(
        proto.STRING,
        number=11,
    )
    quota_increase_eligibility: "QuotaIncreaseEligibility" = proto.Field(
        proto.MESSAGE,
        number=12,
        message="QuotaIncreaseEligibility",
    )
    is_fixed: bool = proto.Field(
        proto.BOOL,
        number=13,
    )
    dimensions_infos: MutableSequence["DimensionsInfo"] = proto.RepeatedField(
        proto.MESSAGE,
        number=14,
        message="DimensionsInfo",
    )
    is_concurrent: bool = proto.Field(
        proto.BOOL,
        number=15,
    )
    service_request_quota_uri: str = proto.Field(
        proto.STRING,
        number=17,
    )


class QuotaIncreaseEligibility(proto.Message):
    r"""Eligibility information regarding requesting increase
    adjustment of a quota.

    Attributes:
        is_eligible (bool):
            Whether a higher quota value can be requested
            for the quota.
        ineligibility_reason (google.cloud.cloudquotas_v1.types.QuotaIncreaseEligibility.IneligibilityReason):
            The reason of why it is ineligible to request increased
            value of the quota. If the is_eligible field is true, it
            defaults to INELIGIBILITY_REASON_UNSPECIFIED.
    """

    class IneligibilityReason(proto.Enum):
        r"""The enumeration of reasons when it is ineligible to request
        increase adjustment.

        Values:
            INELIGIBILITY_REASON_UNSPECIFIED (0):
                Default value when is_eligible is true.
            NO_VALID_BILLING_ACCOUNT (1):
                The container is not linked with a valid
                billing account.
            OTHER (2):
                Other reasons.
        """
        INELIGIBILITY_REASON_UNSPECIFIED = 0
        NO_VALID_BILLING_ACCOUNT = 1
        OTHER = 2

    is_eligible: bool = proto.Field(
        proto.BOOL,
        number=1,
    )
    ineligibility_reason: IneligibilityReason = proto.Field(
        proto.ENUM,
        number=2,
        enum=IneligibilityReason,
    )


class QuotaPreference(proto.Message):
    r"""QuotaPreference represents the preferred quota configuration
    specified for a project, folder or organization. There is only
    one QuotaPreference resource for a quota value targeting a
    unique set of dimensions.

    Attributes:
        name (str):
            Required except in the CREATE requests. The resource name of
            the quota preference. The ID component following
            "locations/" must be "global". Example:
            ``projects/123/locations/global/quotaPreferences/my-config-for-us-east1``
        dimensions (MutableMapping[str, str]):
            Immutable. The dimensions that this quota preference applies
            to. The key of the map entry is the name of a dimension,
            such as "region", "zone", "network_id", and the value of the
            map entry is the dimension value.

            If a dimension is missing from the map of dimensions, the
            quota preference applies to all the dimension values except
            for those that have other quota preferences configured for
            the specific value.

            NOTE: QuotaPreferences can only be applied across all values
            of "user" and "resource" dimension. Do not set values for
            "user" or "resource" in the dimension map.

            Example: {"provider", "Foo Inc"} where "provider" is a
            service specific dimension.
        quota_config (google.cloud.cloudquotas_v1.types.QuotaConfig):
            Required. Preferred quota configuration.
        etag (str):
            Optional. The current etag of the quota
            preference. If an etag is provided on update and
            does not match the current server's etag of the
            quota preference, the request will be blocked
            and an ABORTED error will be returned. See
            https://google.aip.dev/134#etags for more
            details on etags.
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Create time stamp
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Update time stamp
        service (str):
            Required. The name of the service to which
            the quota preference is applied.
        quota_id (str):
            Required. The id of the quota to which the quota preference
            is applied. A quota name is unique in the service. Example:
            ``CpusPerProjectPerRegion``
        reconciling (bool):
            Output only. Is the quota preference pending
            Google Cloud approval and fulfillment.
        justification (str):
            The reason / justification for this quota
            preference.
        contact_email (str):
            Required. Input only. An email address that
            can be used for quota related communication
            between the Google Cloud and the user in case
            the Google Cloud needs further information to
            make a decision on whether the user preferred
            quota can be granted.

            The Google account for the email address must
            have quota update permission for the project,
            folder or organization this quota preference is
            for.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    dimensions: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=2,
    )
    quota_config: "QuotaConfig" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="QuotaConfig",
    )
    etag: str = proto.Field(
        proto.STRING,
        number=4,
    )
    create_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=5,
        message=timestamp_pb2.Timestamp,
    )
    update_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=6,
        message=timestamp_pb2.Timestamp,
    )
    service: str = proto.Field(
        proto.STRING,
        number=7,
    )
    quota_id: str = proto.Field(
        proto.STRING,
        number=8,
    )
    reconciling: bool = proto.Field(
        proto.BOOL,
        number=10,
    )
    justification: str = proto.Field(
        proto.STRING,
        number=11,
    )
    contact_email: str = proto.Field(
        proto.STRING,
        number=12,
    )


class QuotaConfig(proto.Message):
    r"""The preferred quota configuration.

    Attributes:
        preferred_value (int):
            Required. The preferred value. Must be
            greater than or equal to -1. If set to -1, it
            means the value is "unlimited".
        state_detail (str):
            Output only. Optional details about the state
            of this quota preference.
        granted_value (google.protobuf.wrappers_pb2.Int64Value):
            Output only. Granted quota value.
        trace_id (str):
            Output only. The trace id that the Google
            Cloud uses to provision the requested quota.
            This trace id may be used by the client to
            contact Cloud support to track the state of a
            quota preference request. The trace id is only
            produced for increase requests and is unique for
            each request. The quota decrease requests do not
            have a trace id.
        annotations (MutableMapping[str, str]):
            Optional. The annotations map for clients to
            store small amounts of arbitrary data. Do not
            put PII or other sensitive information here. See
            https://google.aip.dev/128#annotations
        request_origin (google.cloud.cloudquotas_v1.types.QuotaConfig.Origin):
            Output only. The origin of the quota
            preference request.
    """

    class Origin(proto.Enum):
        r"""The enumeration of the origins of quota preference requests.

        Values:
            ORIGIN_UNSPECIFIED (0):
                The unspecified value.
            CLOUD_CONSOLE (1):
                Created through Cloud Console.
            AUTO_ADJUSTER (2):
                Generated by automatic quota adjustment.
        """
        ORIGIN_UNSPECIFIED = 0
        CLOUD_CONSOLE = 1
        AUTO_ADJUSTER = 2

    preferred_value: int = proto.Field(
        proto.INT64,
        number=1,
    )
    state_detail: str = proto.Field(
        proto.STRING,
        number=2,
    )
    granted_value: wrappers_pb2.Int64Value = proto.Field(
        proto.MESSAGE,
        number=3,
        message=wrappers_pb2.Int64Value,
    )
    trace_id: str = proto.Field(
        proto.STRING,
        number=4,
    )
    annotations: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=5,
    )
    request_origin: Origin = proto.Field(
        proto.ENUM,
        number=6,
        enum=Origin,
    )


class DimensionsInfo(proto.Message):
    r"""The detailed quota information such as effective quota value
    for a combination of dimensions.

    Attributes:
        dimensions (MutableMapping[str, str]):
            The map of dimensions for this dimensions
            info. The key of a map entry is "region", "zone"
            or the name of a service specific dimension, and
            the value of a map entry is the value of the
            dimension.  If a dimension does not appear in
            the map of dimensions, the dimensions info
            applies to all the dimension values except for
            those that have another DimenisonInfo instance
            configured for the specific value.
            Example: {"provider" : "Foo Inc"} where
            "provider" is a service specific dimension of a
            quota.
        details (google.cloud.cloudquotas_v1.types.QuotaDetails):
            Quota details for the specified dimensions.
        applicable_locations (MutableSequence[str]):
            The applicable regions or zones of this dimensions info. The
            field will be set to ['global'] for quotas that are not per
            region or per zone. Otherwise, it will be set to the list of
            locations this dimension info is applicable to.
    """

    dimensions: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=1,
    )
    details: "QuotaDetails" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="QuotaDetails",
    )
    applicable_locations: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=3,
    )


class QuotaDetails(proto.Message):
    r"""The quota details for a map of dimensions.

    Attributes:
        value (int):
            The value currently in effect and being
            enforced.
    """

    value: int = proto.Field(
        proto.INT64,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from google.cloud.cloudquotas import gapic_version as package_version

__version__ = package_version.__version__


from google.cloud.cloudquotas_v1.services.cloud_quotas.async_client import (
    CloudQuotasAsyncClient,
)
from google.cloud.cloudquotas_v1.services.cloud_quotas.client import CloudQuotasClient
from google.cloud.cloudquotas_v1.types.cloudquotas import (
    CreateQuotaPreferenceRequest,
    GetQuotaInfoRequest,
    GetQuotaPreferenceRequest,
    ListQuotaInfosRequest,
    ListQuotaInfosResponse,
    ListQuotaPreferencesRequest,
    ListQuotaPreferencesResponse,
    UpdateQuotaPreferenceRequest,
)
from google.cloud.cloudquotas_v1.types.resources import (
    DimensionsInfo,
    QuotaConfig,
    QuotaDetails,
    QuotaIncreaseEligibility,
    QuotaInfo,
    QuotaPreference,
    QuotaSafetyCheck,
)

__all__ = (
    "CloudQuotasClient",
    "CloudQuotasAsyncClient",
    "CreateQuotaPreferenceRequest",
    "GetQuotaInfoRequest",
    "GetQuotaPreferenceRequest",
    "ListQuotaInfosRequest",
    "ListQuotaInfosResponse",
    "ListQuotaPreferencesRequest",
    "ListQuotaPreferencesResponse",
    "UpdateQuotaPreferenceRequest",
    "DimensionsInfo",
    "QuotaConfig",
    "QuotaDetails",
    "QuotaIncreaseEligibility",
    "QuotaInfo",
    "QuotaPreference",
    "QuotaSafetyCheck",
)



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from collections import OrderedDict
import functools
import re
from typing import (
    Dict,
    Mapping,
    MutableMapping,
    MutableSequence,
    Optional,
    Sequence,
    Tuple,
    Type,
    Union,
)

from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1
from google.api_core import retry_async as retries
from google.api_core.client_options import ClientOptions
from google.auth import credentials as ga_credentials  # type: ignore
from google.oauth2 import service_account  # type: ignore

from google.cloud.cloudquotas_v1 import gapic_version as package_version

try:
    OptionalRetry = Union[retries.AsyncRetry, gapic_v1.method._MethodDefault, None]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.AsyncRetry, object, None]  # type: ignore

from google.protobuf import field_mask_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore

from google.cloud.cloudquotas_v1.services.cloud_quotas import pagers
from google.cloud.cloudquotas_v1.types import cloudquotas, resources

from .client import CloudQuotasClient
from .transports.base import DEFAULT_CLIENT_INFO, CloudQuotasTransport
from .transports.grpc_asyncio import CloudQuotasGrpcAsyncIOTransport


class CloudQuotasAsyncClient:
    """The Cloud Quotas API is an infrastructure service for Google
    Cloud that lets service consumers list and manage their resource
    usage limits.

    - List/Get the metadata and current status of the quotas for a
      service.
    - Create/Update quota preferencess that declare the preferred
      quota values.
    - Check the status of a quota preference request.
    - List/Get pending and historical quota preference.
    """

    _client: CloudQuotasClient

    # Copy defaults from the synchronous client for use here.
    # Note: DEFAULT_ENDPOINT is deprecated. Use _DEFAULT_ENDPOINT_TEMPLATE instead.
    DEFAULT_ENDPOINT = CloudQuotasClient.DEFAULT_ENDPOINT
    DEFAULT_MTLS_ENDPOINT = CloudQuotasClient.DEFAULT_MTLS_ENDPOINT
    _DEFAULT_ENDPOINT_TEMPLATE = CloudQuotasClient._DEFAULT_ENDPOINT_TEMPLATE
    _DEFAULT_UNIVERSE = CloudQuotasClient._DEFAULT_UNIVERSE

    quota_info_path = staticmethod(CloudQuotasClient.quota_info_path)
    parse_quota_info_path = staticmethod(CloudQuotasClient.parse_quota_info_path)
    quota_preference_path = staticmethod(CloudQuotasClient.quota_preference_path)
    parse_quota_preference_path = staticmethod(
        CloudQuotasClient.parse_quota_preference_path
    )
    common_billing_account_path = staticmethod(
        CloudQuotasClient.common_billing_account_path
    )
    parse_common_billing_account_path = staticmethod(
        CloudQuotasClient.parse_common_billing_account_path
    )
    common_folder_path = staticmethod(CloudQuotasClient.common_folder_path)
    parse_common_folder_path = staticmethod(CloudQuotasClient.parse_common_folder_path)
    common_organization_path = staticmethod(CloudQuotasClient.common_organization_path)
    parse_common_organization_path = staticmethod(
        CloudQuotasClient.parse_common_organization_path
    )
    common_project_path = staticmethod(CloudQuotasClient.common_project_path)
    parse_common_project_path = staticmethod(
        CloudQuotasClient.parse_common_project_path
    )
    common_location_path = staticmethod(CloudQuotasClient.common_location_path)
    parse_common_location_path = staticmethod(
        CloudQuotasClient.parse_common_location_path
    )

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            CloudQuotasAsyncClient: The constructed client.
        """
        return CloudQuotasClient.from_service_account_info.__func__(CloudQuotasAsyncClient, info, *args, **kwargs)  # type: ignore

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            CloudQuotasAsyncClient: The constructed client.
        """
        return CloudQuotasClient.from_service_account_file.__func__(CloudQuotasAsyncClient, filename, *args, **kwargs)  # type: ignore

    from_service_account_json = from_service_account_file

    @classmethod
    def get_mtls_endpoint_and_cert_source(
        cls, client_options: Optional[ClientOptions] = None
    ):
        """Return the API endpoint and client cert source for mutual TLS.

        The client cert source is determined in the following order:
        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
        client cert source is None.
        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
        default client cert source exists, use the default one; otherwise the client cert
        source is None.

        The API endpoint is determined in the following order:
        (1) if `client_options.api_endpoint` if provided, use the provided one.
        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
        default mTLS endpoint; if the environment variable is "never", use the default API
        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
        use the default API endpoint.

        More details can be found at https://google.aip.dev/auth/4114.

        Args:
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. Only the `api_endpoint` and `client_cert_source` properties may be used
                in this method.

        Returns:
            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
                client cert source to use.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
        """
        return CloudQuotasClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore

    @property
    def transport(self) -> CloudQuotasTransport:
        """Returns the transport used by the client instance.

        Returns:
            CloudQuotasTransport: The transport used by the client instance.
        """
        return self._client.transport

    @property
    def api_endpoint(self):
        """Return the API endpoint used by the client instance.

        Returns:
            str: The API endpoint used by the client instance.
        """
        return self._client._api_endpoint

    @property
    def universe_domain(self) -> str:
        """Return the universe domain used by the client instance.

        Returns:
            str: The universe domain used
                by the client instance.
        """
        return self._client._universe_domain

    get_transport_class = functools.partial(
        type(CloudQuotasClient).get_transport_class, type(CloudQuotasClient)
    )

    def __init__(
        self,
        *,
        credentials: Optional[ga_credentials.Credentials] = None,
        transport: Union[str, CloudQuotasTransport] = "grpc_asyncio",
        client_options: Optional[ClientOptions] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the cloud quotas async client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Union[str, ~.CloudQuotasTransport]): The
                transport to use. If set to None, a transport is chosen
                automatically.
            client_options (Optional[Union[google.api_core.client_options.ClientOptions, dict]]):
                Custom options for the client.

                1. The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client when ``transport`` is
                not explicitly provided. Only if this property is not set and
                ``transport`` was not explicitly provided, the endpoint is
                determined by the GOOGLE_API_USE_MTLS_ENDPOINT environment
                variable, which have one of the following values:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto-switch to the
                default mTLS endpoint if client certificate is present; this is
                the default value).

                2. If the GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide a client certificate for mTLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

                3. The ``universe_domain`` property can be used to override the
                default "googleapis.com" universe. Note that ``api_endpoint``
                property still takes precedence; and ``universe_domain`` is
                currently not supported for mTLS.

            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client = CloudQuotasClient(
            credentials=credentials,
            transport=transport,
            client_options=client_options,
            client_info=client_info,
        )

    async def list_quota_infos(
        self,
        request: Optional[Union[cloudquotas.ListQuotaInfosRequest, dict]] = None,
        *,
        parent: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> pagers.ListQuotaInfosAsyncPager:
        r"""Lists QuotaInfos of all quotas for a given project,
        folder or organization.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import cloudquotas_v1

            async def sample_list_quota_infos():
                # Create a client
                client = cloudquotas_v1.CloudQuotasAsyncClient()

                # Initialize request argument(s)
                request = cloudquotas_v1.ListQuotaInfosRequest(
                    parent="parent_value",
                )

                # Make the request
                page_result = client.list_quota_infos(request=request)

                # Handle the response
                async for response in page_result:
                    print(response)

        Args:
            request (Optional[Union[google.cloud.cloudquotas_v1.types.ListQuotaInfosRequest, dict]]):
                The request object. Message for requesting list of
                QuotaInfos
            parent (:class:`str`):
                Required. Parent value of QuotaInfo resources. Listing
                across different resource containers (such as
                'projects/-') is not allowed.

                Example names:
                ``projects/123/locations/global/services/compute.googleapis.com``
                ``folders/234/locations/global/services/compute.googleapis.com``
                ``organizations/345/locations/global/services/compute.googleapis.com``

                This corresponds to the ``parent`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.cloudquotas_v1.services.cloud_quotas.pagers.ListQuotaInfosAsyncPager:
                Message for response to listing
                QuotaInfos
                Iterating over this object will yield
                results and resolve additional pages
                automatically.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([parent])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = cloudquotas.ListQuotaInfosRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if parent is not None:
            request.parent = parent

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.list_quota_infos,
            default_retry=retries.AsyncRetry(
                initial=1.0,
                maximum=10.0,
                multiplier=1.3,
                predicate=retries.if_exception_type(
                    core_exceptions.ServiceUnavailable,
                ),
                deadline=60.0,
            ),
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListQuotaInfosAsyncPager(
            method=rpc,
            request=request,
            response=response,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def get_quota_info(
        self,
        request: Optional[Union[cloudquotas.GetQuotaInfoRequest, dict]] = None,
        *,
        name: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> resources.QuotaInfo:
        r"""Retrieve the QuotaInfo of a quota for a project,
        folder or organization.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import cloudquotas_v1

            async def sample_get_quota_info():
                # Create a client
                client = cloudquotas_v1.CloudQuotasAsyncClient()

                # Initialize request argument(s)
                request = cloudquotas_v1.GetQuotaInfoRequest(
                    name="name_value",
                )

                # Make the request
                response = await client.get_quota_info(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.cloudquotas_v1.types.GetQuotaInfoRequest, dict]]):
                The request object. Message for getting a QuotaInfo
            name (:class:`str`):
                Required. The resource name of the quota info.

                An example name:
                ``projects/123/locations/global/services/compute.googleapis.com/quotaInfos/CpusPerProjectPerRegion``

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.cloudquotas_v1.types.QuotaInfo:
                QuotaInfo represents information
                about a particular quota for a given
                project, folder or organization.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([name])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = cloudquotas.GetQuotaInfoRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.get_quota_info,
            default_retry=retries.AsyncRetry(
                initial=1.0,
                maximum=10.0,
                multiplier=1.3,
                predicate=retries.if_exception_type(
                    core_exceptions.ServiceUnavailable,
                ),
                deadline=60.0,
            ),
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def list_quota_preferences(
        self,
        request: Optional[Union[cloudquotas.ListQuotaPreferencesRequest, dict]] = None,
        *,
        parent: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> pagers.ListQuotaPreferencesAsyncPager:
        r"""Lists QuotaPreferences in a given project, folder or
        organization.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import cloudquotas_v1

            async def sample_list_quota_preferences():
                # Create a client
                client = cloudquotas_v1.CloudQuotasAsyncClient()

                # Initialize request argument(s)
                request = cloudquotas_v1.ListQuotaPreferencesRequest(
                    parent="parent_value",
                )

                # Make the request
                page_result = client.list_quota_preferences(request=request)

                # Handle the response
                async for response in page_result:
                    print(response)

        Args:
            request (Optional[Union[google.cloud.cloudquotas_v1.types.ListQuotaPreferencesRequest, dict]]):
                The request object. Message for requesting list of
                QuotaPreferences
            parent (:class:`str`):
                Required. Parent value of QuotaPreference resources.
                Listing across different resource containers (such as
                'projects/-') is not allowed.

                When the value starts with 'folders' or 'organizations',
                it lists the QuotaPreferences for org quotas in the
                container. It does not list the QuotaPreferences in the
                descendant projects of the container.

                Example parents: ``projects/123/locations/global``

                This corresponds to the ``parent`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.cloudquotas_v1.services.cloud_quotas.pagers.ListQuotaPreferencesAsyncPager:
                Message for response to listing
                QuotaPreferences
                Iterating over this object will yield
                results and resolve additional pages
                automatically.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([parent])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = cloudquotas.ListQuotaPreferencesRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if parent is not None:
            request.parent = parent

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.list_quota_preferences,
            default_retry=retries.AsyncRetry(
                initial=1.0,
                maximum=10.0,
                multiplier=1.3,
                predicate=retries.if_exception_type(
                    core_exceptions.ServiceUnavailable,
                ),
                deadline=60.0,
            ),
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListQuotaPreferencesAsyncPager(
            method=rpc,
            request=request,
            response=response,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def get_quota_preference(
        self,
        request: Optional[Union[cloudquotas.GetQuotaPreferenceRequest, dict]] = None,
        *,
        name: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> resources.QuotaPreference:
        r"""Gets details of a single QuotaPreference.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import cloudquotas_v1

            async def sample_get_quota_preference():
                # Create a client
                client = cloudquotas_v1.CloudQuotasAsyncClient()

                # Initialize request argument(s)
                request = cloudquotas_v1.GetQuotaPreferenceRequest(
                    name="name_value",
                )

                # Make the request
                response = await client.get_quota_preference(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.cloudquotas_v1.types.GetQuotaPreferenceRequest, dict]]):
                The request object. Message for getting a QuotaPreference
            name (:class:`str`):
                Required. Name of the resource

                Example name:
                ``projects/123/locations/global/quota_preferences/my-config-for-us-east1``

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.cloudquotas_v1.types.QuotaPreference:
                QuotaPreference represents the
                preferred quota configuration specified
                for a project, folder or organization.
                There is only one QuotaPreference
                resource for a quota value targeting a
                unique set of dimensions.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([name])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = cloudquotas.GetQuotaPreferenceRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.get_quota_preference,
            default_retry=retries.AsyncRetry(
                initial=1.0,
                maximum=10.0,
                multiplier=1.3,
                predicate=retries.if_exception_type(
                    core_exceptions.ServiceUnavailable,
                ),
                deadline=60.0,
            ),
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def create_quota_preference(
        self,
        request: Optional[Union[cloudquotas.CreateQuotaPreferenceRequest, dict]] = None,
        *,
        parent: Optional[str] = None,
        quota_preference: Optional[resources.QuotaPreference] = None,
        quota_preference_id: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> resources.QuotaPreference:
        r"""Creates a new QuotaPreference that declares the
        desired value for a quota.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import cloudquotas_v1

            async def sample_create_quota_preference():
                # Create a client
                client = cloudquotas_v1.CloudQuotasAsyncClient()

                # Initialize request argument(s)
                quota_preference = cloudquotas_v1.QuotaPreference()
                quota_preference.quota_config.preferred_value = 1595
                quota_preference.service = "service_value"
                quota_preference.quota_id = "quota_id_value"
                quota_preference.contact_email = "contact_email_value"

                request = cloudquotas_v1.CreateQuotaPreferenceRequest(
                    parent="parent_value",
                    quota_preference=quota_preference,
                )

                # Make the request
                response = await client.create_quota_preference(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.cloudquotas_v1.types.CreateQuotaPreferenceRequest, dict]]):
                The request object. Message for creating a
                QuotaPreference
            parent (:class:`str`):
                Required. Value for parent.

                Example: ``projects/123/locations/global``

                This corresponds to the ``parent`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            quota_preference (:class:`google.cloud.cloudquotas_v1.types.QuotaPreference`):
                Required. The resource being created
                This corresponds to the ``quota_preference`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            quota_preference_id (:class:`str`):
                Optional. Id of the requesting
                object, must be unique under its parent.
                If client does not set this field, the
                service will generate one.

                This corresponds to the ``quota_preference_id`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.cloudquotas_v1.types.QuotaPreference:
                QuotaPreference represents the
                preferred quota configuration specified
                for a project, folder or organization.
                There is only one QuotaPreference
                resource for a quota value targeting a
                unique set of dimensions.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([parent, quota_preference, quota_preference_id])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = cloudquotas.CreateQuotaPreferenceRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if parent is not None:
            request.parent = parent
        if quota_preference is not None:
            request.quota_preference = quota_preference
        if quota_preference_id is not None:
            request.quota_preference_id = quota_preference_id

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.create_quota_preference,
            default_retry=retries.AsyncRetry(
                initial=1.0,
                maximum=10.0,
                multiplier=1.3,
                predicate=retries.if_exception_type(
                    core_exceptions.ServiceUnavailable,
                ),
                deadline=60.0,
            ),
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def update_quota_preference(
        self,
        request: Optional[Union[cloudquotas.UpdateQuotaPreferenceRequest, dict]] = None,
        *,
        quota_preference: Optional[resources.QuotaPreference] = None,
        update_mask: Optional[field_mask_pb2.FieldMask] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> resources.QuotaPreference:
        r"""Updates the parameters of a single QuotaPreference.
        It can updates the config in any states, not just the
        ones pending approval.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import cloudquotas_v1

            async def sample_update_quota_preference():
                # Create a client
                client = cloudquotas_v1.CloudQuotasAsyncClient()

                # Initialize request argument(s)
                quota_preference = cloudquotas_v1.QuotaPreference()
                quota_preference.quota_config.preferred_value = 1595
                quota_preference.service = "service_value"
                quota_preference.quota_id = "quota_id_value"
                quota_preference.contact_email = "contact_email_value"

                request = cloudquotas_v1.UpdateQuotaPreferenceRequest(
                    quota_preference=quota_preference,
                )

                # Make the request
                response = await client.update_quota_preference(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.cloudquotas_v1.types.UpdateQuotaPreferenceRequest, dict]]):
                The request object. Message for updating a
                QuotaPreference
            quota_preference (:class:`google.cloud.cloudquotas_v1.types.QuotaPreference`):
                Required. The resource being updated
                This corresponds to the ``quota_preference`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            update_mask (:class:`google.protobuf.field_mask_pb2.FieldMask`):
                Optional. Field mask is used to specify the fields to be
                overwritten in the QuotaPreference resource by the
                update. The fields specified in the update_mask are
                relative to the resource, not the full request. A field
                will be overwritten if it is in the mask. If the user
                does not provide a mask then all fields will be
                overwritten.

                This corresponds to the ``update_mask`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.cloudquotas_v1.types.QuotaPreference:
                QuotaPreference represents the
                preferred quota configuration specified
                for a project, folder or organization.
                There is only one QuotaPreference
                resource for a quota value targeting a
                unique set of dimensions.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([quota_preference, update_mask])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = cloudquotas.UpdateQuotaPreferenceRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if quota_preference is not None:
            request.quota_preference = quota_preference
        if update_mask is not None:
            request.update_mask = update_mask

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.update_quota_preference,
            default_retry=retries.AsyncRetry(
                initial=1.0,
                maximum=10.0,
                multiplier=1.3,
                predicate=retries.if_exception_type(
                    core_exceptions.ServiceUnavailable,
                ),
                deadline=60.0,
            ),
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("quota_preference.name", request.quota_preference.name),)
            ),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def __aenter__(self) -> "CloudQuotasAsyncClient":
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.transport.close()


DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=package_version.__version__
)


__all__ = ("CloudQuotasAsyncClient",)
